/* indybot - Copyright (C) 2004 bakunin - <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <indybot.h>

int fd, check_time, check_write;
pthread_mutex_t mutex;

#ifdef HAVE_LIBSSL
SSL_CTX *ctx;
SSL *ssl;
SSL_METHOD *meth;
X509 *server_cert;
#endif

char *file_cfg = NULL;
int debug = 0;

char *nick = NULL;
char *server = NULL;
char *password = NULL;
char *charset = NULL;
int port;
int timer;
int secure;
int set_quit = 0;

struct channel *channels = NULL;

#ifdef HAVE_LIBRECODE
RECODE_REQUEST request;
#endif

int
main (int argc, char *argv[])
{
  cfg_t *cfg;
  register int i, c;
  struct channel *chan;

  static cfg_opt_t channel_opts[] = {
    CFG_STR_LIST ("File", 0, CFGF_NONE),
    CFG_END ()
  };

  static cfg_opt_t opts[] = {
    CFG_STR ("Nick", "indybot", CFGF_NONE),
    CFG_STR ("Server", NULL, CFGF_NONE),
    CFG_INT ("Port", 6667, CFGF_NONE),
    CFG_BOOL ("Ssl", cfg_false, CFGF_NONE),
    CFG_STR ("Password", NULL, CFGF_NONE),
    CFG_INT ("Timer", 1, CFGF_NONE),
    CFG_STR ("Charset", "utf-8", CFGF_NONE),
    CFG_SEC ("Channel", channel_opts, CFGF_MULTI | CFGF_TITLE),
    CFG_END ()
  };

#ifdef HAVE_LIBRECODE
  RECODE_OUTER outer = recode_new_outer (true);
  request = recode_new_request (outer);
#endif

  signal (SIGPIPE, SIG_IGN);

  for (i = 1; i < argc; i++)
    {
      if (!strcmp (argv[i], "-f"))
	{

	  i++;
	  if (argv[i])
	    file_cfg = argv[i];

	  else
	    fatal ("Error: argoment to '-f' is missing.");

	}
      else if (!strcmp (argv[i], "-d"))
	{
	  debug = 1;

	}
      else if (!strcmp (argv[i], "-h"))
	{
	  usage ();

	}
      else
	fatal ("Error: unrecognized option.");
    }

  if (!file_cfg)
    file_cfg = FILE_CFG;

  cfg = cfg_init (opts, CFGF_NOCASE);

  switch (cfg_parse (cfg, file_cfg))
    {

    case CFG_FILE_ERROR:
      fatal ("Error: no config files.");

    case CFG_PARSE_ERROR:
      fatal ("Error: file not recognized.");
    }

  if (cfg_getstr (cfg, "Nick") && !(nick = strdup (cfg_getstr (cfg, "Nick"))))
    fatal ("Error: memory.");

  if (cfg_getstr (cfg, "Server")
      && !(server = strdup (cfg_getstr (cfg, "Server"))))
    fatal ("Error: memory.");

  if (cfg_getstr (cfg, "Charset")
      && !(charset = strdup (cfg_getstr (cfg, "Charset"))))
    fatal ("Error: memory.");

  if (!nick || !server || !charset)
    fatal ("Error: no nick or no server or no charset in config file.");

  port = cfg_getint (cfg, "Port");
  timer = cfg_getint (cfg, "Timer");
  secure = cfg_getbool (cfg, "Ssl");

#ifndef HAVE_LIBSSL
  if (secure)
    fatal ("No support for ssl. Compile with openssl.");
#endif

  if ((password = cfg_getstr (cfg, "Password"))
      && !(password = strdup (password)))
    fatal ("No memory.");

  if (!(i = cfg_size (cfg, "Channel")))
    fatal ("Error: no channel in config files.");

  while (i--)
    {
      cfg_t *pl;

      pl = cfg_getnsec (cfg, "Channel", i);

      c = cfg_size (pl, "File");
      if (!c)
	fatal ("Channel %s has no files!", cfg_title (pl));

      chan = add_channel (cfg_title (pl));
      for (c--; c >= 0; c--)
	add_channel_file (chan, cfg_getnstr (pl, "File", c));
    }

  cfg_free (cfg);

  if (!debug)
    {
      if (fork ())
	exit (0);

      setsid ();
    }

  if (pthread_mutex_init (&mutex, NULL))
    fatal ("Error: mutex!");

#ifdef HAVE_LIBSSL
  if (secure)
    SSL_library_init ();
#endif

  while (1)
    {
      irc_socket ();
      sleep (1);
    }

  pthread_mutex_destroy (&mutex);

#ifdef HAVE_LIBRECODE
  recode_delete_request (request);
  recode_delete_outer (outer);
#endif

  return 0;
}

static int
html_trim_check (char *tmp)
{
  int i = 0;

  while (*tmp)
    {
      if (*tmp == '>')
	return i;
      tmp++;
      i++;
    }

  return 0;
}

char *
html_trim (char *tmp)
{
  int i, j, l, len;
  char *ret;

  tmp = trim (tmp);
  len = strlen (tmp);

  puts ("---");
  puts (tmp);

  ret = malloc (sizeof (char) * (len + 1));

  for (i = j = 0; i < len; i++)
    {
      if (tmp[i] != '<' || !(l = html_trim_check (tmp + i)))
	ret[j++] = tmp[i];
      else
	i += l;
    }

  ret[j] = 0;
  free (tmp);

  puts ("***");
  puts (ret);
  puts ("!!!");

  return ret;
}

char *
trim (char *tmp)
{
  int i = 0;
  int len, j;
  char *ret;
  int q, ok = 0;

  while (tmp[i] == ' ' || tmp[i] == '\t' || tmp[i] == '\r' || tmp[i] == '\n')
    tmp++;

  i = strlen (tmp);
  i--;

  while (tmp[i] == ' ' || tmp[i] == '\t' || tmp[i] == '\r' || tmp[i] == '\n')
    i--;

  tmp[i + 1] = 0;

  len = strlen (tmp);

  if (len > 256)
    {
      ok = 1;
      len = 256;
    }

  ret = malloc (sizeof (char) * (len + 1 + (ok ? 3 : 0)));

  for (i = j = q = 0; i < len; i++)
    {
      if (tmp[i] == '\t' || tmp[i] == ' ' || tmp[i] == '\v' || tmp[i] == '\r'
	  || tmp[i] == '\n')
	{
	  if (!q)
	    {
	      q = 1;
	      ret[j++] = ' ';
	    }
	}
      else
	{
	  q = 0;
	  ret[j++] = tmp[i];
	}
    }

  if (ok)
    {
      ret[j++] = '.';
      ret[j++] = '.';
      ret[j++] = '.';
    }

  ret[j++] = 0;

  tmp = strdup (ret);
  free (ret);

  return tmp;
}

void
fatal (const char *str, ...)
{
  va_list va;

  va_start (va, str);
  vfprintf (stderr, str, va);
  va_end (va);

  exit (1);
}

/* EOF */
